@@ -1,11 +1,13 @@
 use {
     crate::{
         accounts_data_meter::AccountsDataMeter,
+        compute_budget::ComputeBudget,
         ic_logger_msg, ic_msg,
         instruction_recorder::InstructionRecorder,
         log_collector::LogCollector,
         native_loader::NativeLoader,
         pre_account::PreAccount,
+        sysvar_cache::SysvarCache,
         timings::{ExecuteDetailsTimings, ExecuteTimings},
     },
     solana_measure::measure::Measure,
@@ -13,7 +15,6 @@ use {
         account::{AccountSharedData, ReadableAccount},
         account_utils::StateMut,
         bpf_loader_upgradeable::{self, UpgradeableLoaderState},
-        compute_budget::ComputeBudget,
         feature_set::{
             cap_accounts_data_len, do_support_realloc, neon_evm_compute_budget,
             reject_empty_instruction_without_program, remove_native_loader, requestable_heap_size,
@@ -26,9 +27,8 @@ use {
         pubkey::Pubkey,
         rent::Rent,
         saturating_add_assign,
-        sysvar::Sysvar,
     },
-    std::{cell::RefCell, collections::HashMap, fmt::Debug, rc::Rc, sync::Arc},
+    std::{borrow::Cow, cell::RefCell, collections::HashMap, fmt::Debug, rc::Rc, sync::Arc},
 };
 
 pub type TransactionAccountRefCell = (Pubkey, Rc<RefCell<AccountSharedData>>);
@@ -194,7 +194,7 @@ pub struct InvokeContext<'a> {
     pre_accounts: Vec<PreAccount>,
     accounts: &'a [TransactionAccountRefCell],
     builtin_programs: &'a [BuiltinProgram],
-    pub sysvars: &'a [(Pubkey, Vec<u8>)],
+    pub sysvar_cache: Cow<'a, SysvarCache>,
     log_collector: Option<Rc<RefCell<LogCollector>>>,
     compute_budget: ComputeBudget,
     current_compute_budget: ComputeBudget,
@@ -215,7 +215,7 @@ impl<'a> InvokeContext<'a> {
         rent: Rent,
         accounts: &'a [TransactionAccountRefCell],
         builtin_programs: &'a [BuiltinProgram],
-        sysvars: &'a [(Pubkey, Vec<u8>)],
+        sysvar_cache: Cow<'a, SysvarCache>,
         log_collector: Option<Rc<RefCell<LogCollector>>>,
         compute_budget: ComputeBudget,
         executors: Rc<RefCell<Executors>>,
@@ -230,7 +230,7 @@ impl<'a> InvokeContext<'a> {
             pre_accounts: Vec::new(),
             accounts,
             builtin_programs,
-            sysvars,
+            sysvar_cache,
             log_collector,
             current_compute_budget: compute_budget,
             compute_budget,
@@ -254,7 +254,7 @@ impl<'a> InvokeContext<'a> {
             Rent::default(),
             accounts,
             builtin_programs,
-            &[],
+            Cow::Owned(SysvarCache::default()),
             Some(LogCollector::new_ref()),
             ComputeBudget::default(),
             Rc::new(RefCell::new(Executors::default())),
@@ -950,21 +950,9 @@ impl<'a> InvokeContext<'a> {
         &self.current_compute_budget
     }
 
-    /// Get the value of a sysvar by its id
-    pub fn get_sysvar<T: Sysvar>(&self, id: &Pubkey) -> Result<T, InstructionError> {
-        self.sysvars
-            .iter()
-            .find_map(|(key, data)| {
-                if id == key {
-                    bincode::deserialize(data).ok()
-                } else {
-                    None
-                }
-            })
-            .ok_or_else(|| {
-                ic_msg!(self, "Unable to get sysvar {}", id);
-                InstructionError::UnsupportedSysvar
-            })
+    /// Get cached sysvars
+    pub fn get_sysvar_cache(&self) -> &SysvarCache {
+        &self.sysvar_cache
     }
 }
 
@@ -1069,7 +1057,7 @@ pub fn mock_process_instruction_with_sysvars(
     mut program_indices: Vec<usize>,
     instruction_data: &[u8],
     keyed_accounts: &[(bool, bool, Pubkey, Rc<RefCell<AccountSharedData>>)],
-    sysvars: &[(Pubkey, Vec<u8>)],
+    sysvar_cache: &SysvarCache,
     process_instruction: ProcessInstructionWithContext,
 ) -> Result<(), InstructionError> {
     let mut preparation =
@@ -1078,7 +1066,7 @@ pub fn mock_process_instruction_with_sysvars(
     program_indices.insert(0, preparation.accounts.len());
     preparation.accounts.push((*loader_id, processor_account));
     let mut invoke_context = InvokeContext::new_mock(&preparation.accounts, &[]);
-    invoke_context.sysvars = sysvars;
+    invoke_context.sysvar_cache = Cow::Borrowed(sysvar_cache);
     invoke_context.push(
         &preparation.message,
         &preparation.message.instructions()[0],
@@ -1100,7 +1088,7 @@ pub fn mock_process_instruction(
         program_indices,
         instruction_data,
         keyed_accounts,
-        &[],
+        &SysvarCache::default(),
         process_instruction,
     )
 }
